Package: FactoMineR
Version: 2.15
Title: Multivariate Exploratory Data Analysis and Data Mining
Authors@R: c(person(given = "Francois", family = "Husson", role = c("aut", "cre"), email = "francois.husson@institut-agro.fr",
             comment = c(ORCID = "0000-0002-7271-8877")),
			 person(given = "Julie", family = "Josse", role = c("aut")),
             person(given = "Sebastien", family = "Le", role = c("aut")),
             person(given = "Jeremy", family = "Mazet", role = c("aut"))
			 )
Maintainer: Francois Husson <francois.husson@institut-agro.fr>
Depends: R (>= 4.3)
Imports:
        car,cluster,DT,ellipse,emmeans,flashClust,graphics,grDevices,lattice,leaps,MASS,multcompView,scatterplot3d,stats,utils,ggplot2,ggrepel,irlba
Suggests: missMDA,knitr,Factoshiny,markdown
Description: Exploratory data analysis methods to summarize, visualize and describe datasets. The main principal component methods are available, those with the largest potential in terms of applications: principal component analysis (PCA) when variables are quantitative, correspondence analysis (CA) and multiple correspondence analysis (MCA) when variables are categorical, Multiple Factor Analysis when variables are structured in groups, etc. and hierarchical cluster analysis. F. Husson, S. Le and J. Pages (2017).
License: GPL (>= 2)
URL: http://factominer.free.fr
BugReports: https://github.com/husson/FactoMineR/issues
Encoding: UTF-8
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-06-03 14:37:44 UTC; husson
Author: Francois Husson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7271-8877>),
  Julie Josse [aut],
  Sebastien Le [aut],
  Jeremy Mazet [aut]
Repository: CRAN
Date/Publication: 2026-06-11 09:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-11 10:56:30 UTC; unix
Archs: FactoMineR.so.dSYM
