\name{tab.disjonctif}

\alias{tab.disjonctif}

\title{Make a disjonctif table}

\description{
Make a disjonctif table.
}

\usage{
tab.disjonctif(tab)
}

\arguments{
  \item{tab}{a data frame with factors}
}

\value{The disjonctif table}

\keyword{algebra}
