\name{FactoMineR-package}
\alias{FactoMineR-package}
\alias{FactoMineR}
\docType{package}
\title{Multivariate Exploratory Data Analysis and Data Mining with R
}
\description{
The method proposed in this package are exploratory mutlivariate methods such as principal component analysis, correspondence analysis or clustering.
}
\details{
FactoMineR is a package for exploratory multivariate data analysis. The package Factoshiny gives an interface to use most of the functions of FactoMineR.
}

\author{
Francois Husson, Julie Josse, Sebastien Le, Jeremy Mazet

Maintainer: \email{francois.husson@institut-agro.fr}
}

\references{
Le, S., Josse, J. & Husson, F. (2008). FactoMineR: An R Package for Multivariate Analysis. Journal of Statistical Software. 25(1). pp. 1-18.
\url{https://www.jstatsoft.org/v25/i01/}

A website: \url{http://factominer.free.fr/}

Some videos: \url{https://www.youtube.com/playlist?list=PLnZgp6epRBbTsZEFXi_p6W48HhNyqwxIu}
}
\keyword{Factor analysis}

